"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var http_1 = require("@angular/common/http");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var utils_1 = require("./utils");
var BaseService = /** @class */ (function () {
    function BaseService(http) {
        this.http = http;
        this.headers = new http_1.HttpHeaders({ 'Content-Type': 'application/json' });
        lastActiveTime = new Date();
    }
    /**
     * 分页查询
     * 参数：url URL地址, queryParam：用于查询的参数。waitLoading: 是否要显示等待加载（缺省true）
     */
    BaseService.prototype.searchPage = function (url, queryParam, waitLoading) {
        var _this = this;
        if (waitLoading === void 0) { waitLoading = true; }
        if (waitLoading) {
            isRunning++;
        }
        if (!queryParam.pageNumber) {
            queryParam.pageNumber = 1;
        }
        if (!queryParam.pageSize) {
            queryParam.pageSize = 10;
        }
        if (url.indexOf("?") < 0) {
            url += "?x=1";
        }
        url += this.toQueryString(queryParam, "");
        return this.http.get(url).pipe(operators_1.tap(function (response) {
            if (waitLoading) {
                isRunning--;
            }
            lastActiveTime = new Date();
            var rst = response;
            _this.processDate(rst.datas);
        }), operators_1.catchError(function (err) { return _this.handleError(err, waitLoading); }));
    };
    /**
     * 把输入的Object转换成请求参数串。
     * 如 obj = {a:1, b:[{i:3, k:4},{i:5}]，则返回的请求串为：
     * "&a=1&b[0].i=3&b[0].k=4&b[1].i=6"
     *
     */
    BaseService.prototype.toQueryString = function (obj, propPrefix) {
        var url = "";
        var preParam = propPrefix;
        if (preParam) {
            preParam += ".";
        }
        if (obj == undefined) {
            return url;
        }
        if (typeof obj == 'object') {
            if (obj instanceof Array) {
                for (var i = 0; i < obj.length; i++) {
                    if (typeof (obj[i]) == 'object') {
                        url = url + this.toQueryString(obj[i], propPrefix + "[" + i + "]");
                    }
                    else {
                        url = url + "&" + propPrefix + "[" + i + "]" + "=" + this.formatValue(obj[i]);
                    }
                }
            }
            else {
                for (var prop in obj) {
                    if (prop != "constructor") {
                        var pv = obj[prop];
                        if (pv) {
                            if (typeof pv == 'function')
                                continue;
                            if (typeof pv == 'object') {
                                url = url + this.toQueryString(pv, preParam + prop);
                            }
                            else {
                                url = url + "&" + preParam + prop + "=" + this.formatValue(obj[prop]);
                            }
                        }
                    }
                }
            }
        }
        else {
            url = url + "&" + propPrefix + "=" + this.formatValue(obj);
        }
        return url;
    };
    BaseService.prototype.formatValue = function (val) {
        if (val instanceof Date) {
            return val.getTime();
        }
        return encodeURI(val);
    };
    /**
     * GET请求
     * @param url URL地址
     * @param data：用于查询的参数
     * @param waitLoading: 是否要显示等待加载（缺省true）
     */
    BaseService.prototype.doGet = function (url, data, waitLoading) {
        var _this = this;
        if (waitLoading === void 0) { waitLoading = true; }
        if (data) {
            if (url.indexOf("?") < 0) {
                url += "?a=1";
            }
        }
        url += this.toQueryString(data, "");
        if (waitLoading) {
            isRunning++;
        }
        return this.http.get(url)
            .pipe(operators_1.tap(function (resp) {
            if (waitLoading) {
                isRunning--;
            }
            lastActiveTime = new Date();
            _this.processDate(resp);
        }), operators_1.catchError(function (err) { return _this.handleError(err, waitLoading); }));
    };
    /**
     * PUT请求
     * @param url URL地址,
     * @param data：发送到服务器的数据
     * @param waitLoading: 是否要显示等待加载（缺省true）
     */
    BaseService.prototype.doPut = function (url, data, waitLoading) {
        var _this = this;
        if (waitLoading === void 0) { waitLoading = true; }
        if (waitLoading) {
            isRunning++;
        }
        return this.http
            .put(url, data, { headers: this.headers })
            .pipe(operators_1.tap(function (resp) {
            if (waitLoading) {
                isRunning--;
            }
            lastActiveTime = new Date();
            _this.processDate(resp);
        }), operators_1.catchError(function (err) { return _this.handleError(err, waitLoading); }));
    };
    /**
     * POST请求
     * @param url URL地址
     * @param data：要post的数据
     * @param waitLoading: 是否要显示等待加载（缺省true）
     *
     */
    BaseService.prototype.doPost = function (url, data, waitLoading) {
        var _this = this;
        if (waitLoading === void 0) { waitLoading = true; }
        if (waitLoading) {
            isRunning++;
        }
        return this.http
            .post(url, JSON.stringify(data), { headers: this.headers })
            .pipe(operators_1.tap(function (response) {
            if (waitLoading) {
                isRunning--;
            }
            lastActiveTime = new Date();
            _this.processDate(response);
            return response;
        }), operators_1.catchError(function (err) { return _this.handleError(err, waitLoading); }));
    };
    /**
     * DELETE请求
     * @param url URL地址,
     * @param waitLoading: 是否要显示等待加载（缺省true）
     */
    BaseService.prototype.doDelete = function (url, waitLoading) {
        var _this = this;
        if (waitLoading === void 0) { waitLoading = true; }
        if (waitLoading) {
            isRunning++;
        }
        return this.http.delete(url, { headers: this.headers })
            .pipe(operators_1.tap(function (response) { return _this.processResponse(response, waitLoading); }), operators_1.catchError(function (err) { return _this.handleError(err, waitLoading); }));
    };
    BaseService.prototype.processResponse = function (response, waitLoading) {
        if (waitLoading) {
            isRunning--;
        }
        lastActiveTime = new Date();
        return this.processDate(response);
    };
    BaseService.prototype.processDate = function (rst) {
        if (rst) {
            if (Array.isArray(rst)) {
                for (var _i = 0, rst_1 = rst; _i < rst_1.length; _i++) {
                    var r = rst_1[_i];
                    this.processDate(r);
                }
            }
            else {
                for (var p in rst) {
                    if (typeof (rst[p]) == "object") {
                        this.processDate(rst[p]);
                    }
                    else if (p.endsWith("Date") || p.endsWith("Time") || p.endsWith("Timestamp")) {
                        if (typeof (rst[p]) == "number") {
                            rst[p] = new Date(rst[p]);
                        }
                        else if (rst[p].length == 28 && rst[p].indexOf("T") == 10) { //"2017-11-29T12:18:54.921+0000"
                            rst[p] = new Date(rst[p]);
                        }
                    }
                }
            }
        }
        return rst;
    };
    BaseService.prototype.handleError = function (error, waitLoading) {
        //console.error('An error occurred:'+ error);
        if (waitLoading) {
            isRunning--;
        }
        lastActiveTime = new Date();
        var msg = utils_1.Utils.resolveJsonErrorMessage(error);
        return rxjs_1.throwError(msg);
    };
    return BaseService;
}());
exports.BaseService = BaseService;
//# sourceMappingURL=base.service.js.map