export declare class GraphUtils {
    /**
     * 把节点集和边集转换成图矩阵
     * @param nodes 节点集
     * @param edges 边集
     * @param directed 是否有向，缺省是无向图
     * @param distanceProp 边上的距离属性名，缺省为distance
     * @return 图矩阵
     */
    static toGraphMatrix(nodes: NodeBase<any>[], edges: EdgeBase<any>[], directed?: boolean, distanceProp?: string): number[][];
    private static getMatrix;
    /** 从某一源点出发，找到到所有结点的最短路径
     *
     * @param nodes 节点集合
     * @param edges 边集合
     * @param start 起点
     * @param directed 是否有向图，缺省为无向
     */
    static getShortedPathAll(nodes: NodeBase<any>[], edges: EdgeBase<any>[], start: NodeBase<any>, directed?: boolean): NodeBase<any>[][];
    /** 从某一源点出发，找到到某一结点的最短路径
     *
     * @param nodes 节点集合
     * @param edges 边集合
     * @param start 起点
     * @param end 终点
     * @param directed 是否有向图，缺省为无向
     */
    static getShortedPath(nodes: NodeBase<any>[], edges: EdgeBase<any>[], start: NodeBase<any>, end: NodeBase<any>, directed?: boolean): NodeBase<any>[];
    /**
     * 找到与指定节点相连的节点
     * @param nodes 节点集
     * @param edges 边集
     * @param start 指定节点
     * @param directed 是否有向图，缺省是无向图
     */
    static getConnectedNodes(nodes: NodeBase<any>[], edges: EdgeBase<any>[], start: NodeBase<any>, directed?: boolean): NodeBase<any>[];
    /**
     * 找到指定节点在节点集合中的位置
     * @param nodes 节点集合
     * @param v0 指定节点
     * @return 位置，0-based. 如果没找到就返回 undefined;
     */
    static getNodePosition(nodes: NodeBase<any>[], v0: NodeBase<any>): number;
    /**
     * 判断从一个节点出发，到另一个节点是否连通
     * @param nodes 所有点集
     * @param edges 所有边集
     * @param start 起点
     * @param end 终点
     * @param directed 是否有向图
     * @return 是否连通
     */
    static isConnected(nodes: NodeBase<any>[], edges: EdgeBase<any>[], start: NodeBase<any>, end: NodeBase<any>, directed: boolean): boolean;
    static getConectedbyBfs(graph: number[][], start: number): number[];
    static isConectedbyBfs(arr: number[][], start: number, end: number): boolean;
    /**
     * 把业务点和关联数据转换为图的点和关联数据
     * @param nodes 业务节点集合，如客户集合
     * @param edges 业务关联集合，如客户关系集合
     * @param idProp 可选参数，业务节点ID的属性名，如客户代码“code”，缺省为"id"
     * @param sourceIdProp 可选参数，业务关联的源ID属性名，如如客户关联关系中的“sourceCustCode”，缺省为"sourceId"
     * @param targetIdProp 可选参数，业务关联的目标ID属性名，如如客户关联关系中的“targetCustCode”，缺省为"targetId"
     * @return 位置，0-based. 如果没找到就返回 undefined;
     */
    static toGraphData<N, E>(nodes: Array<N>, edges: Array<E>, idProp?: string, sourceIdProp?: string, targetIdProp?: string): GraphData<N, E>;
}
/**
 * 图相关数据模型，节点基类。
 */
export declare class NodeBase<N> {
    /**
     * 节点的ID
     */
    id: string;
    /**
     * 节点的业务数据，如Customer
     */
    data: N;
}
/**
 * 图相关数据模型，连接节点的边基类
 */
export declare class EdgeBase<E> {
    /**
     * 源节点的ID
     */
    source: NodeBase<any>;
    /**
     * 目标节点的ID
     */
    target: NodeBase<any>;
    /**
     * 源节点到目标节点的距离，用于计算最短路径
     */
    distance: number;
    /**
     * 边相关的业务数据，如CustomerRelation
     */
    data: E;
}
export declare class GraphData<N, E> {
    nodes: NodeBase<N>[];
    edges: EdgeBase<E>[];
}
