"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var dijkstra_utils_1 = require("./dijkstra.utils");
var GraphUtils = /** @class */ (function () {
    function GraphUtils() {
    }
    /**
     * 把节点集和边集转换成图矩阵
     * @param nodes 节点集
     * @param edges 边集
     * @param directed 是否有向，缺省是无向图
     * @param distanceProp 边上的距离属性名，缺省为distance
     * @return 图矩阵
     */
    GraphUtils.toGraphMatrix = function (nodes, edges, directed, distanceProp) {
        if (directed === void 0) { directed = false; }
        if (distanceProp === void 0) { distanceProp = "distance"; }
        var m = GraphUtils.getMatrix(nodes.length);
        var nodeMap = new Object();
        var idx = 0;
        for (var _i = 0, nodes_1 = nodes; _i < nodes_1.length; _i++) {
            var vertex = nodes_1[_i];
            nodeMap[vertex.id] = idx;
            idx++;
        }
        for (var _a = 0, edges_1 = edges; _a < edges_1.length; _a++) {
            var edge = edges_1[_a];
            var distance = edge[distanceProp];
            if (!distance) {
                distance = 1;
            }
            m[nodeMap[edge.source.id]][nodeMap[edge.target.id]] = distance;
            if (!directed) {
                m[nodeMap[edge.target.id]][nodeMap[edge.source.id]] = distance;
            }
        }
        return m;
    };
    GraphUtils.getMatrix = function (length) {
        var m = new Array(length);
        for (var idx = 0; idx < m.length; idx++) {
            m[idx] = new Array(length);
        }
        return m;
    };
    /** 从某一源点出发，找到到所有结点的最短路径
     *
     * @param nodes 节点集合
     * @param edges 边集合
     * @param start 起点
     * @param directed 是否有向图，缺省为无向
     */
    GraphUtils.getShortedPathAll = function (nodes, edges, start, directed) {
        if (directed === void 0) { directed = false; }
        var graph = GraphUtils.toGraphMatrix(nodes, edges, directed);
        var pos = GraphUtils.getNodePosition(nodes, start);
        var paths = GraphUtils.getMatrix(nodes.length);
        var dists = dijkstra_utils_1.Dijkstra.getShortedPathAll(graph, pos, paths);
        var pathNodeMatrix = new Array();
        for (var _i = 0, paths_1 = paths; _i < paths_1.length; _i++) {
            var path = paths_1[_i];
            var pathNodes = new Array();
            for (var _a = 0, path_1 = path; _a < path_1.length; _a++) {
                var n = path_1[_a];
                if (n == undefined) {
                    break;
                }
                pathNodes.push(nodes[n]);
            }
            pathNodeMatrix.push(pathNodes);
        }
        return pathNodeMatrix;
    };
    /** 从某一源点出发，找到到某一结点的最短路径
     *
     * @param nodes 节点集合
     * @param edges 边集合
     * @param start 起点
     * @param end 终点
     * @param directed 是否有向图，缺省为无向
     */
    GraphUtils.getShortedPath = function (nodes, edges, start, end, directed) {
        if (directed === void 0) { directed = false; }
        var graph = GraphUtils.toGraphMatrix(nodes, edges, directed);
        var startPos = GraphUtils.getNodePosition(nodes, start);
        var endPos = GraphUtils.getNodePosition(nodes, end);
        var path = new Array(nodes.length);
        var dist = dijkstra_utils_1.Dijkstra.getShortedPath(graph, startPos, endPos, path);
        var pathNodes = new Array();
        for (var _i = 0, path_2 = path; _i < path_2.length; _i++) {
            var n = path_2[_i];
            if (!n) {
                break;
            }
            pathNodes.push(nodes[n]);
        }
        return pathNodes;
    };
    /**
     * 找到与指定节点相连的节点
     * @param nodes 节点集
     * @param edges 边集
     * @param start 指定节点
     * @param directed 是否有向图，缺省是无向图
     */
    GraphUtils.getConnectedNodes = function (nodes, edges, start, directed) {
        if (directed === void 0) { directed = false; }
        var graph = GraphUtils.toGraphMatrix(nodes, edges, directed);
        var pos = GraphUtils.getNodePosition(nodes, start);
        var m = GraphUtils.getMatrix(nodes.length);
        var nodeIdxs = GraphUtils.getConectedbyBfs(graph, pos);
        var cverts = [];
        for (var _i = 0, nodeIdxs_1 = nodeIdxs; _i < nodeIdxs_1.length; _i++) {
            var i = nodeIdxs_1[_i];
            cverts.push(nodes[i]);
        }
        graph = null;
        m = null;
        nodeIdxs = null;
        return cverts;
    };
    /**
     * 找到指定节点在节点集合中的位置
     * @param nodes 节点集合
     * @param v0 指定节点
     * @return 位置，0-based. 如果没找到就返回 undefined;
     */
    GraphUtils.getNodePosition = function (nodes, v0) {
        for (var i = 0; i < nodes.length; i++) {
            if (nodes[i].id == v0.id) {
                return i;
            }
        }
        return undefined;
    };
    /**
     * 判断从一个节点出发，到另一个节点是否连通
     * @param nodes 所有点集
     * @param edges 所有边集
     * @param start 起点
     * @param end 终点
     * @param directed 是否有向图
     * @return 是否连通
     */
    GraphUtils.isConnected = function (nodes, edges, start, end, directed) {
        var graph = GraphUtils.toGraphMatrix(nodes, edges, directed);
        var startPos = GraphUtils.getNodePosition(nodes, start);
        var endPos = GraphUtils.getNodePosition(nodes, end);
        return GraphUtils.isConectedbyBfs(graph, startPos, endPos);
    };
    GraphUtils.getConectedbyBfs = function (graph, start) {
        var row = graph.length;
        var queue = [];
        var i = start;
        var visited = {}; //记录遍历顺序
        var order = []; //记录顺序，给自己看的
        queue.push(i); //先把根节点加入
        while (queue.length) { //如果队列没有被清空，也就是还没遍历完毕
            for (var j = 0; j < row; j++) {
                if (graph[i][j]) { //如果是1
                    if (!visited[j]) {
                        queue.push(j); //队列加入未访问
                    }
                }
            }
            queue.shift(); //取出队列第一个
            visited[i] = true; //记录已经访问
            while (visited[queue[0]]) {
                queue.shift();
            }
            order.push(i); //记录顺序
            i = queue[0];
        }
        queue = null;
        visited = null;
        return order;
    };
    GraphUtils.isConectedbyBfs = function (arr, start, end) {
        var row = arr.length;
        var queue = [];
        var i = start;
        var visited = {}; //记录遍历顺序
        var order = []; //记录顺序，给自己看的
        queue.push(i); //先把根节点加入
        while (queue.length) { //如果队列没有被清空，也就是还没遍历完毕
            for (var j = 0; j < row; j++) {
                if (arr[i][j]) { //如果是1
                    if (!visited[j]) {
                        queue.push(j); //队列加入未访问
                    }
                }
            }
            queue.shift(); //取出队列第一个
            visited[i] = true; //记录已经访问
            console.log("visit node:" + i);
            if (i == end) {
                return true;
            }
            while (visited[queue[0]]) {
                queue.shift();
            }
            order.push(i); //记录顺序
            i = queue[0];
        }
        queue = null;
        visited = null;
        order = null;
        return false;
    };
    /**
     * 把业务点和关联数据转换为图的点和关联数据
     * @param nodes 业务节点集合，如客户集合
     * @param edges 业务关联集合，如客户关系集合
     * @param idProp 可选参数，业务节点ID的属性名，如客户代码“code”，缺省为"id"
     * @param sourceIdProp 可选参数，业务关联的源ID属性名，如如客户关联关系中的“sourceCustCode”，缺省为"sourceId"
     * @param targetIdProp 可选参数，业务关联的目标ID属性名，如如客户关联关系中的“targetCustCode”，缺省为"targetId"
     * @return 位置，0-based. 如果没找到就返回 undefined;
     */
    GraphUtils.toGraphData = function (nodes, edges, idProp, sourceIdProp, targetIdProp) {
        if (idProp === void 0) { idProp = "id"; }
        if (sourceIdProp === void 0) { sourceIdProp = "sourceId"; }
        if (targetIdProp === void 0) { targetIdProp = "targetId"; }
        var nodeMap = {};
        var graphData = new GraphData();
        graphData.nodes = new Array(nodes.length);
        graphData.edges = new Array(edges.length);
        for (var i = 0; i < nodes.length; i++) {
            var nd = nodes[i];
            var node = new NodeBase();
            node.id = nd[idProp];
            node.data = nd;
            nodeMap[node.id] = node;
            graphData.nodes[i] = node;
        }
        for (var i = 0; i < edges.length; i++) {
            var ed = edges[i];
            var edge = new EdgeBase();
            edge.source = nodeMap[ed[sourceIdProp]];
            edge.target = nodeMap[ed[targetIdProp]];
            edge.data = ed;
            graphData.edges[i] = edge;
        }
        nodeMap = null;
        return graphData;
    };
    return GraphUtils;
}());
exports.GraphUtils = GraphUtils;
/**
 * 图相关数据模型，节点基类。
 */
var NodeBase = /** @class */ (function () {
    function NodeBase() {
    }
    return NodeBase;
}());
exports.NodeBase = NodeBase;
/**
 * 图相关数据模型，连接节点的边基类
 */
var EdgeBase = /** @class */ (function () {
    function EdgeBase() {
        /**
         * 源节点到目标节点的距离，用于计算最短路径
         */
        this.distance = 1;
    }
    return EdgeBase;
}());
exports.EdgeBase = EdgeBase;
var GraphData = /** @class */ (function () {
    function GraphData() {
    }
    return GraphData;
}());
exports.GraphData = GraphData;
//# sourceMappingURL=graph.utils.js.map